# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import logging
import os

from task_processing.interfaces.task_executor import make_task_config
from task_processing.plugins.mesos.mesos_executor import MesosExecutor
from task_processing.runners.sync import Sync

logging.basicConfig()


def main():
    mesos_address = os.environ.get('MESOS', '127.0.0.1:5050')
    executor = MesosExecutor(
        credential_secret_file="/src/task_processing/examples/cluster/secret",
        mesos_address=mesos_address
    )
    task_config = make_task_config(image="ubuntu:14.04", cmd="/bin/sleep 10")
    runner = Sync(executor)
    result = runner.run(task_config)
    print(result)
    print(result.original_event)
    runner.stop()


if __name__ == "__main__":
    exit(main())



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import logging
import os

from six.moves.queue import Empty
from six.moves.queue import Queue

from task_processing.events.event import EventTerminal
from task_processing.interfaces.task_executor import make_task_config
from task_processing.plugins.mesos.mesos_executor import MesosExecutor
from task_processing.runners.subscription import Subscription

logging.basicConfig()


def main():
    credentials = {'principal': 'mesos', 'secret': 'very'}
    mesos_address = os.environ['MESOS']
    executor = MesosExecutor(credentials=credentials,
                             mesos_address=mesos_address)
    queue = Queue(100)
    runner = Subscription(executor, queue)

    tasks = set()
    for _ in range(1, 100):
        task_config = make_task_config(image="ubuntu:14.04", cmd="/bin/sleep 10")
        tasks.add(task_config.task_id)
        runner.run(task_config)

    print("Running {} tasks: {}".format(len(tasks), tasks))
    while len(tasks) > 0:
        try:
            event = queue.get(True, 10)
        except Empty:
            event = None

        if event is None:
            print("Timeout on subscription queue, still waiting for {}".format(tasks))
        else:
            print("{} {}".format(event.task_id, type(event)))
            if isinstance(event, EventTerminal):
                tasks.discard(event.task_id)

    runner.stop()


if __name__ == "__main__":
    exit(main())



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

from threading import Thread

from six.moves.queue import Empty
from six.moves.queue import Full

from task_processing.interfaces.runner import Runner


class Subscription(Runner):
    def __init__(self, executor, queue):
        self.executor = executor
        self.event_queue = queue
        self.stopping = False
        self.producer_t = Thread(target=self.event_producer)
        self.producer_t.daemon = True
        self.producer_t.start()

    def event_producer(self):
        executor_queue = self.executor.get_event_queue()
        while True:
            if self.stopping:
                return
            try:
                event = executor_queue.get(True, 1)
                self.event_queue.put(event, False)
            except Empty:
                pass
            except Full:
                pass

    def run(self, task_config):
        return self.executor.run(task_config)

    def kill(self, task_id):
        return self.executor.kill(task_id)

    def stop(self):
        self.executor.stop()
        self.stopping = True
        self.producer_t.join()



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import logging
import os
import time

from task_processing.events.event import EventTerminal
from task_processing.interfaces.task_executor import make_task_config
from task_processing.plugins.mesos.mesos_executor import MesosExecutor
from task_processing.runners.async import Async

logging.basicConfig()


class Counter(object):
    def __init__(self):
        self.terminated = 0

    def process_event(self, event):
        print('{} {}'.format(event.task_id, type(event)))
        self.terminated += 1


def main():
    mesos_address = os.environ['MESOS']
    executor = MesosExecutor(
        credential_secret_file="/src/task_processing/examples/cluster/secret",
        mesos_address=mesos_address
    )

    counter = Counter()
    runner = Async(executor, [(EventTerminal, counter.process_event)])

    for _ in range(100):
        task_config = make_task_config(image="ubuntu:14.04",
                                       cmd="/bin/sleep 10")
        runner.run(task_config)

    while True:
        print('terminated {} tasks'.format(counter.terminated))
        if counter.terminated >= 100:
            return
        time.sleep(10)

    runner.stop()


if __name__ == "__main__":
    exit(main())



from __future__ import absolute_import
from __future__ import unicode_literals

from task_processing.events import event

# https://github.com/apache/mesos/blob/master/include/mesos/mesos.proto
MESOS_TASK_STATUS_TO_EVENT = {
    0: event.EventStarting,
    1: event.EventRunning,
    2: event.EventFinished,
    3: event.EventFailed,
    4: event.EventKilled,
    5: event.EventLost,
    6: event.EventStaging,
    7: event.EventError,
    8: event.EventKilling,
    9: event.EventDropped,
    10: event.EventUnreachable,
    11: event.EventGone,
    12: event.EventGoneByOperator,
    13: event.EventUnknown
}


def mesos_status_to_event(mesos_status):
    return MESOS_TASK_STATUS_TO_EVENT[mesos_status.state](mesos_status)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

from threading import Thread

from six.moves.queue import Empty

from task_processing.interfaces.runner import Runner


class AsyncError(Exception):
    pass


class Async(Runner):
    def __init__(self, executor, callbacks=None):
        if not callbacks:
            raise AsyncError("must provide at least one callback")

        self.callbacks = callbacks
        self.executor = executor
        self.stopping = False

        callback_t = Thread(target=self.callback_loop)
        callback_t.daemon = True
        callback_t.start()

    def run(self, task_config):
        return self.executor.run(task_config)

    def kill(self, task_config):
        pass

    def callback_loop(self):
        event_queue = self.executor.get_event_queue()

        while True:
            if self.stopping:
                return

            try:
                event = event_queue.get(True, 10)

                for (cl, fn) in self.callbacks:
                    if isinstance(event, cl):
                        fn(event)
            except Empty:
                pass

    def stop(self):
        self.executor.stop()
        self.stopping = True
        self.callback_t.join()
